'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _includes = require('lodash/includes');

var _includes2 = _interopRequireDefault(_includes);

var _extraEntryWebpackPlugin = require('extra-entry-webpack-plugin');

var _extraEntryWebpackPlugin2 = _interopRequireDefault(_extraEntryWebpackPlugin);

var _readMultipleFiles = require('read-multiple-files');

var _readMultipleFiles2 = _interopRequireDefault(_readMultipleFiles);

var _emitAssets = require('./utils/emitAssets');

var _emitAssets2 = _interopRequireDefault(_emitAssets);

var _registerPlugins = require('./registerPlugins');

var _registerPlugins2 = _interopRequireDefault(_registerPlugins);

var _registerDefaultPlugins = require('./registerDefaultPlugins');

var _registerDefaultPlugins2 = _interopRequireDefault(_registerDefaultPlugins);

var _createHTML = require('./utils/createHTML');

var _createHTML2 = _interopRequireDefault(_createHTML);

var _getCommonsChunkFilename = require('./utils/getCommonsChunkFilename');

var _getCommonsChunkFilename2 = _interopRequireDefault(_getCommonsChunkFilename);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * apply.js
 *
 * This is the meat of our webpack plugin, it's what does all the component
 * finding and compiling
 */

function apply(compiler) {
  var _this = this;

  var dest = this.options.dest;
  var filter = this.options.filter;

  // Register either the plugins or the default plugins
  if (this.options.plugins && this.options.plugins.length > 0) {
    (0, _registerPlugins2.default)(compiler, this.options.plugins);
  } else {
    (0, _registerDefaultPlugins2.default)(compiler);
  }

  // Compile the client
  var userBundleFileName = _path2.default.join(dest, 'user-bundle.js');
  var userEntries = compiler.options.entry;
  var devServerOptions = compiler.options.devServer;
  var commonsChunkFilename = (0, _getCommonsChunkFilename2.default)(compiler.options.plugins);

  // Load the dynamic resolve loader with a placeholder file
  var extraEntries = ['!!' + require.resolve('./dynamic-resolve.js') + '?' + JSON.stringify({
    filter: filter.toString(),
    componentRoot: this.options.componentRoot,
    context: compiler.context,
    dest: this.options.dest,
    commonsChunkFilename: commonsChunkFilename
  }) + '!' + require.resolve('./assets/placeholder.js')];
  // Find out if we need to include the webpack-dev-server client
  // TODO Test automatically if the user has any variant (middlware, devserver,...) of HMR enabled
  var devServerWithHMR = (0, _includes2.default)(userEntries, 'webpack-dev-server/client') && devServerOptions && devServerOptions.hot;
  var middlwareWithHMR = (0, _includes2.default)(userEntries, 'webpack-hot-middleware/client');

  if (this.options.hot !== false && (this.options.hot === true || devServerWithHMR)) {
    if ((0, _includes2.default)(userEntries, 'webpack/hot/only-dev-server')) {
      extraEntries.unshift('webpack/hot/only-dev-server');
    }
    extraEntries.unshift('webpack-dev-server/client?http://' + devServerOptions.host + ':' + devServerOptions.port);
  } else if (this.options.hot !== false && (this.options.hot === true || middlwareWithHMR)) {
    if ((0, _includes2.default)(userEntries, 'webpack/hot/only-dev-server')) {
      extraEntries.unshift('webpack/hot/only-dev-server');
    }
    extraEntries.unshift('webpack-hot-middleware/client');
  }
  // Apply the ExtraEntry plugin with our entries above, a unique entryName
  // and ouput everything to userBundleFileName
  compiler.apply(new _extraEntryWebpackPlugin2.default({
    entry: extraEntries,
    entryName: 'CarteBlanche_' + this.id,
    outputName: userBundleFileName
  }));

  // The client assets, default the HTML to only include the client bundles and the
  // user bundle
  var clientAssets = {
    'index.html': (0, _createHTML2.default)({ dest: dest, commonsChunkFilename: commonsChunkFilename }),
    'client-bundle.js': _fs2.default.readFileSync(_path2.default.resolve(__dirname, './assets/client-bundle.js')),
    'client-bundle.css': _fs2.default.readFileSync(_path2.default.resolve(__dirname, './assets/client.css')),
    'iframe-client-bundle.js': _fs2.default.readFileSync(_path2.default.resolve(__dirname, './assets/iframe-client-bundle.js'))
  };

  compiler.plugin('emit', function (compilation, callback) {
    // If some custom files were passed by the user, default to them
    var assets = _this.options.files || [];
    // Allow plugin developers to add assets to the client
    compilation.applyPlugins('carte-blanche-plugin-assets-processing', assets);
    // If any custom assets were passed in, read the files from the filesystem
    if (assets.length > 0) {
      (0, _readMultipleFiles2.default)(assets, function (err, contents) {
        if (err) {
          throw err;
        }
        var scripts = [];
        var styles = [];
        // Depending on the asset type that was passed add them to a script
        // or style tag
        assets.forEach(function (assetFilename, index) {
          switch (assetFilename.substr(-3)) {
            case '.js':
              scripts.push(contents[index]);break;
            case 'css':
              styles.push(contents[index]);break;
            default:
              break;
          }
        });
        // Put together the HTML file based on the assets we got
        clientAssets['index.html'] = (0, _createHTML2.default)({
          dest: dest,
          extraScripts: scripts,
          extraStyles: styles
        });
        (0, _emitAssets2.default)(compilation, clientAssets, dest, callback);
      });
    } else {
      // If not custom assets were passed in by neither the user nor any plugins
      // emit the defaults straight away
      (0, _emitAssets2.default)(compilation, clientAssets, dest, callback);
    }
  });

  // Don't add the carte-blanche chunk to html files
  compiler.plugin('compilation', function (compilation) {
    return compilation.plugin('html-webpack-plugin-alter-chunks', function (chunks) {
      return chunks.filter(function (chunk) {
        return chunk.files.indexOf(userBundleFileName) === -1;
      });
    });
  });

  // Log out that CarteBlanche has started
  if (devServerOptions && devServerOptions.host && devServerOptions.port) {
    // eslint-disable-next-line no-console
    console.log('CarteBlanche started at http://' + devServerOptions.host + ':' + devServerOptions.port + '/' + dest + '!');
  } else {
    // eslint-disable-next-line no-console
    console.log('CarteBlanche started at /' + dest);
  }
}

exports.default = apply;