'use strict';

var _loaderUtils = require('loader-utils');

var _loaderUtils2 = _interopRequireDefault(_loaderUtils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Every component is run through this loader, and the plugins and data specified by
 * plugin developers are properly generated and emitted
 *
 * @param  {String} source The raw, unchanged source code of the component
 */
module.exports = function pluginsLoader(source) {
  // Flag this loader as cacheable to webpack
  this.cacheable();
  var query = _loaderUtils2.default.parseQuery(this.query);

  var pluginPromises = [];
  /**
   * Define an api which can be called by the plugins in the
   * carte-blanche-plugin-processing step
   */
  function renderToClient(pluginOptions) {
    // Make the plugin name mandatory
    if (!pluginOptions || !pluginOptions.name) {
      throw new Error('Plugin name is mandatory');
    }

    // Load the frontend data and pass it to the frontend part of the plugin
    pluginPromises.push(new Promise(function (resolve, reject) {
      Promise.resolve(pluginOptions.frontendData).then(function (frontendData) {
        return resolve({
          name: pluginOptions.name,
          frontendData: frontendData,
          frontendPlugin: pluginOptions.frontendPlugin
        });
      }).catch(function (err) {
        return reject(err);
      });
    }));
  }

  // Pass the source code of the component to the plugins
  var pluginData = { source: source, dest: query.dest, commonsChunkFilename: query.commonsChunkFilename };

  // Trigger events for carte-blanche child plugins
  this._compilation.applyPlugins( // eslint-disable-line no-underscore-dangle
  'carte-blanche-plugin-before-processing', pluginData, this);

  this._compilation.applyPlugins( // eslint-disable-line no-underscore-dangle
  'carte-blanche-plugin-processing', renderToClient, pluginData, this);

  // The callback sends webpack the result of this loader. By using this.async(),
  // we tell webpack that this loader will asynchronously return the result.
  // See http://mxs.is/goognh for more information.
  var callback = this.async();
  // Wait for all plugins to be loaded
  Promise.all(pluginPromises).then(function (plugins) {
    // eslint-disable-line arrow-body-style
    // Get the component data from each plugin
    return plugins.map(function (plugin) {
      // Execute the default export of the plugin frontend module
      var frontendCode = 'function() {\n            return (require(' + JSON.stringify(plugin.frontendPlugin) + '))\n              .default.apply(\n                this,\n                Array.prototype.concat.apply([this.frontendData, pluginData], arguments)\n              )\n          }';
      // Return the data of the plugin
      return '{\n            name: ' + JSON.stringify(plugin.name) + ',\n            frontendData: ' + JSON.stringify(plugin.frontendData) + ',\n            frontendPlugin: ' + (plugin.frontendPlugin ? frontendCode : '""') + '\n          }';
    });
  })
  // Send the data of all the plugins to the next loader via the callback
  .then(function (pluginsData) {
    callback(null, '\n        var pluginData = ' + JSON.stringify(pluginData) + ';\n        module.exports = [' + pluginsData.join(',') + '];\n        ');
  }).catch(function (err) {
    return callback(err);
  });
};